@@ -1,5 +1,7 @@
 import base64
 from datetime import datetime
+from langdetect import detect
+
 import os
 from google.protobuf.json_format import Parse, ParseDict,MessageToDict
 import nltk
@@ -16,7 +18,7 @@ class ModelTool:
         if isinstance(model_root_dir,str) is False:
             raise ValueError("model_root_dir is not str")
         if os.path.exists(model_root_dir) is False:
-            raise ValueError("model_root_dir is not exist")
+            raise ValueError(f"model_root_dir {model_root_dir} is not exist")
         self.__common_tool = CommonTool()
         self.__logger = self.__common_tool.get_logger()
         self.__model_management_file = os.path.join(os.path.join(self.__common_tool.get_project_directory(),"resources"),"model_management.json")
@@ -30,6 +32,11 @@ class ModelTool:
         self.__model_name_to_infer_method["bert"] = self.bert_calculate_embedding
         self.__model_root_dir = model_root_dir
         self.__latest_support_package_number = set([1000,5000,10000])
+        self.__keyword_sortinfo_package_recent_article_number = 1000
+        self.__keyword_sortinfo_package_top_k = 100
+        self.__default_bucket = "gpu-model-data"
+        self.__current_keyword_support_set =set(['en','zh-cn'])
+
 
     
     def get_valid_model_and_version(self):
@@ -108,10 +115,13 @@ class ModelTool:
         self.valid_model_name_and_version(model_name,model_version)
         self.valid_infer_document(dict_document)
         bert_embedding_tool = BertEmbedding()
-        id_to_infer_result = dict()
-        for current_id, current_text in dict_document.items():
-            id_to_infer_result[current_id] = bert_embedding_tool.calculate_embedding(current_text)      
-        return id_to_infer_result 
+        if model_version == "v1":
+            return bert_embedding_tool.calculate_batch_document_embeddings(dict_document)
+        elif model_version == "v2":
+            result_dict = dict()
+            for current_entry_id,current_document  in dict_document.items():
+                result_dict[current_entry_id] = bert_embedding_tool.calculate_document_embeddings_with_split_document(current_document)
+            return result_dict
         
     def __validate_model_management_file(self,path):
         if isinstance(path,str) is False:
@@ -302,7 +312,6 @@ class ModelTool:
         return all_feed_id_to_feed
         
 
-    
     def download_latest_article_embedding_package(self,model_name,model_version,latest_number,publish_time=None):
         """_summary_
 
@@ -426,14 +435,7 @@ class ModelTool:
                     filter_url_to_embedding_dict[current_url] = url_to_embedding_dict[current_url]
             return filter_url_to_article_dict,filter_url_to_embedding_dict
         else:
-            return url_to_article_dict,url_to_embedding_dict
-        
-            
-            
-            
-                
-        
-        
+            return url_to_article_dict,url_to_embedding_dict  
     
     def init_model(self,model_name,model_version):
         nltk.download('stopwords')
@@ -518,5 +520,292 @@ class ModelTool:
                 return False
         return True
                 
+    def download_list_increment_package(self,model_name,model_version,list_package):
+        if isinstance(list_package,list) is False:
+            raise ValueError("list_package is not list")
+        for current_package in list_package:
+            if isinstance(current_package,str) is False:
+                raise ValueError("current_package is not str")
+        self.valid_model_name_and_version(model_name,model_version)
+        
+        merge_url_to_article_dict = dict()
+        merge_url_to_embedding_dict = dict()
+        for current_package in list_package:
+            url_to_article_dict,url_to_embedding_dict = self.download_increment_package(model_name,model_version,current_package)
+            merge_url_to_article_dict.update(url_to_article_dict)
+            merge_url_to_embedding_dict.update(url_to_embedding_dict)
+            
+        return merge_url_to_article_dict,merge_url_to_embedding_dict
+        
+        
+        
+    def download_increment_package(self,model_name,model_version,package_key,publish_time=None):
+        """_summary_
+        download increment embedding package
+        Args:
+            model_name (_type_): _description_
+            model_version (_type_): _description_
+            latest_number (_type_): _description_
+            start_time (_type_, optional): _description_. Defaults to None.
+
+        Raises:
+            ValueError: _description_
+            ValueError: _description_
+            ValueError: _description_
+            ValueError: _description_
+            ValueError: _description_
+
+        Returns:
+            _type_: _description_
+            url_to_article_dict 
+            {
+                "http...":{
+                    "url":
+                    "full_text":
+                    "created_at":
+                    "published_at":
+                    "title":
+                    "author":  may not exist
+                    "content":  may not exist
+                    "feed_id":
+                    "hash": string
+                    "image_url": string
+                    "keyword_list": list string
+                    "cloud_id": int
+                    "major_language": string
+                    
+                }
+                "http...":{
+                    
+                }
+            }
+            
+            url_to_embedding
+            {
+                "https://":{
+                    "url":
+                    "model_name":
+                    "model_version":
+                    "embeddings":[] list float
+                }
+            }
+        """
+        if isinstance(package_key,str) is False:
+            raise ValueError("package_key is not str")
+        article_field_set = set(['url', 'full_text', 'created_at', 'published_at', 'title', 'author', 'content', 'feed_id', 'hash','image_url','keyword_list','cloud_id','major_language'])
+        
+        self.valid_model_name_and_version(model_name,model_version)
+
+        latest_package_key = package_key
+        model_version_embedding_dir = os.path.join(os.path.join(os.path.join(self.__model_root_dir,model_name),model_version),'embedding')
+        if os.path.exists(model_version_embedding_dir) is False or os.path.isdir(model_version_embedding_dir) is False:
+            os.makedirs(model_version_embedding_dir)
+        current_embedding_path = os.path.join(model_version_embedding_dir,latest_package_key)
+                        
+        current_model_detail = self.__model_dict[model_name][model_version]
+        current_bucket_name = current_model_detail['s3_bucket']
+        
+        need_redownload = False
+        if os.path.exists(current_embedding_path) is False:
+            self.__logger.debug(f'current_embedding_path {current_embedding_path}  not exist')
+            need_redownload = True
+        else:
+            exist_protobuf_compress_hash = self.__common_tool.calculate_md5_for_big_file(current_embedding_path)
+            self.__logger.debug(f'current_embedding_path {current_embedding_path} does  exist ,exist file hash {exist_protobuf_compress_hash}')
+            response_header = self.__awss3tool.get_object_header(current_bucket_name, latest_package_key)
+            if ("ResponseMetadata" not in response_header or 
+                "HTTPStatusCode" not in response_header["ResponseMetadata"] or
+                response_header["ResponseMetadata"]["HTTPStatusCode"] != 200):
+                raise ValueError(f'current_bucket { current_bucket_name} key {latest_package_key} not exist')
+            if response_header["Metadata"]["md5_digest"] != exist_protobuf_compress_hash:
+                need_redownload = True
                 
-                
\ No newline at end of file
+        if need_redownload:
+            result = self.__awss3tool.get_object_byte(current_bucket_name,latest_package_key)
+            if result["success"] is False:
+                raise ValueError(f"download embedding package fail result {result}")
+            current_package_compress_byte = result["bytes"]
+            with open(current_embedding_path,'wb') as f:
+                f.write(current_package_compress_byte)
+            self.__logger.debug(f'need_download {current_embedding_path}')
+        else:
+            with open(current_embedding_path,'rb') as f:
+                current_package_compress_byte = f.read()
+        
+        decompress_bytes = zlib.decompress(current_package_compress_byte)
+        current_latest_package = rec_proto_embebding.LatestPackage()
+        current_latest_package.ParseFromString(decompress_bytes)
+        
+        article_embedding_dict = MessageToDict(current_latest_package,preserving_proto_field_name=True)
+        article_list = article_embedding_dict["articles"]
+        embedding_list = article_embedding_dict["embeddings"]
+        url_to_article_dict = dict()
+        url_to_embedding_dict = dict()
+        for current_article in article_list:
+            current_article['published_at'] = int(current_article['published_at'])
+            current_article['created_at'] = int(current_article['created_at'])
+            for current_field in article_field_set:
+                if current_field not in current_article:
+                    current_article[current_field] = None
+                
+            url_to_article_dict[current_article["url"]] = current_article
+            
+        for current_embedding in embedding_list:
+            url_to_embedding_dict[current_embedding["url"]] = current_embedding    
+        
+        
+        if publish_time is not None: 
+            start_time_timestamp = int(round(publish_time.timestamp() * 1000))       
+            filter_url_to_article_dict = dict()
+            filter_url_to_embedding_dict = dict()
+            for current_url,current_article in url_to_article_dict.items():
+
+                if current_article['published_at'] > start_time_timestamp:
+                    filter_url_to_article_dict[current_url] = current_article
+                    filter_url_to_embedding_dict[current_url] = url_to_embedding_dict[current_url]
+            return filter_url_to_article_dict,filter_url_to_embedding_dict
+        else:
+            return url_to_article_dict,url_to_embedding_dict
+        
+    def get_keyword_sortinfo_package_name(self,recent_article_number,top_k,language):
+        if isinstance(recent_article_number,int) is False:
+            raise ValueError("recent_article_number is not int")
+        if recent_article_number < 1:
+            raise ValueError(f"recent_article_number {recent_article_number} is not positive")
+        if isinstance(top_k,int) is False:
+            raise ValueError("top_k is not int")
+        if top_k < 1:
+            raise ValueError(f"top_k {top_k} is not positive")
+        if isinstance(language,str) is False:
+            raise ValueError("language is not str")
+        if language not in self.__current_keyword_support_set:
+            raise ValueError(f"language {language} is not support")
+        package_name = f'keyword_sortinfo_package_recent_article_{recent_article_number}_tok_k_{top_k}_language_{language}'
+        return package_name
+    
+    def get_keyword_support_set(self):
+        return self.__current_keyword_support_set
+    
+    def download_keyword_sortinfo_package(self,major_language):
+        """_summary_
+        """
+        if isinstance(major_language,str) is False:
+            raise ValueError("major_language is not str")
+        
+        package_key = self.get_keyword_sortinfo_package_name(1000,100,major_language)
+        # article_field_set = set(['url', 'full_text', 'created_at', 'published_at', 'title', 'author', 'content', 'feed_id', 'hash','image_url'])
+    
+        keyword_sortinfo_package_dir = os.path.join(self.__model_root_dir, 'sortinfo_package')
+        if os.path.exists(keyword_sortinfo_package_dir) is False or os.path.isdir(keyword_sortinfo_package_dir) is False:
+            os.makedirs(keyword_sortinfo_package_dir)
+        current_sortinfo_package_path = os.path.join(keyword_sortinfo_package_dir,package_key)
+                        
+        current_bucket_name = self.__default_bucket
+        
+        need_redownload = False
+        if os.path.exists(current_sortinfo_package_path) is False:
+            self.__logger.debug(f'current_sortinfo_package_path {current_sortinfo_package_path}  not exist')
+            need_redownload = True
+        else:
+            exist_protobuf_compress_hash = self.__common_tool.calculate_md5_for_big_file(current_sortinfo_package_path)
+            self.__logger.debug(f'current_embedding_path {current_sortinfo_package_path} does  exist ,exist file hash {exist_protobuf_compress_hash}')
+            response_header = self.__awss3tool.get_object_header(current_bucket_name, package_key)
+            if ("ResponseMetadata" not in response_header or 
+                "HTTPStatusCode" not in response_header["ResponseMetadata"] or
+                response_header["ResponseMetadata"]["HTTPStatusCode"] != 200):
+                raise ValueError(f'current_bucket { current_bucket_name} key {package_key} not exist')
+            if response_header["Metadata"]["md5_digest"] != exist_protobuf_compress_hash:
+                need_redownload = True
+                
+        if need_redownload:
+            result = self.__awss3tool.get_object_byte(current_bucket_name,package_key)
+            if result["success"] is False:
+                raise ValueError(f"download embedding package fail result {result}")
+            current_package_compress_byte = result["bytes"]
+            with open(current_sortinfo_package_path,'wb') as f:
+                f.write(current_package_compress_byte)
+            self.__logger.debug(f'need_download {current_sortinfo_package_path}')
+        else:
+            with open(current_sortinfo_package_path,'rb') as f:
+                current_package_compress_byte = f.read()
+        
+        decompress_bytes = zlib.decompress(current_package_compress_byte)
+        current_latest_package = rec_proto_embebding.KeywordSortedInfoPackage()
+        current_latest_package.ParseFromString(decompress_bytes)
+        
+        keyword_sortinfo_dict = MessageToDict(current_latest_package,preserving_proto_field_name=True)
+        return keyword_sortinfo_dict["keyword_sortinfo_list"]
+    
+    def infer_text_language_type(self,text):
+        """
+        af, ar, bg, bn, ca, cs, cy, da, de, el, en, es, et, fa, fi, fr, gu, he,
+        hi, hr, hu, id, it, ja, kn, ko, lt, lv, mk, ml, mr, ne, nl, no, pa, pl,
+        pt, ro, ru, sk, sl, so, sq, sv, sw, ta, te, th, tl, tr, uk, ur, vi, zh-cn, zh-tw
+        https://en.wikipedia.org/wiki/List_of_ISO_639-1_codes
+        """
+        if isinstance(text,str) is False:
+            raise ValueError("text is not str")
+        language_type = None
+        try:
+            language_type = detect(text)
+        except Exception as ex:
+            self.__logger.debug(f'language_type ex: {str(ex)}')
+        return language_type
+        
+
+    def download_latest_all_category(self):
+        '''
+        {
+            "category_name":{
+                "category":"",
+                "level":"first",
+                "subcategory_list":[] first level have
+                "parent":"" second level have
+            }
+        }
+        '''
+        latest_package_key = 'all_category'
+        common_dir = os.path.join(self.__model_root_dir,'common')
+        if os.path.exists(common_dir) is False or os.path.isdir(common_dir) is False:
+            os.makedirs(common_dir)
+        current_feed_path = os.path.join(common_dir,latest_package_key)
+        current_bucket_name = 'gpu-model-data'
+                           
+        need_redownload = False
+        if os.path.exists(latest_package_key) is False:
+            self.__logger.debug(f'current_all_feeds_path {current_feed_path}  not exist')
+            need_redownload = True
+        else:
+            exist_protobuf_compress_hash = self.__common_tool.calculate_md5_for_big_file(current_feed_path)
+            self.__logger.debug(f'current_feed_path {current_feed_path} does  exist ,exist file hash {exist_protobuf_compress_hash}')
+            response_header = self.__awss3tool.get_object_header(current_bucket_name, latest_package_key)
+            if ("ResponseMetadata" not in response_header or 
+                "HTTPStatusCode" not in response_header["ResponseMetadata"] or
+                response_header["ResponseMetadata"]["HTTPStatusCode"] != 200):
+                raise ValueError(f'current_bucket { current_bucket_name} key {latest_package_key} not exist')
+            if response_header["Metadata"]["md5_digest"] != exist_protobuf_compress_hash:
+                need_redownload = True
+                
+        if need_redownload:
+            result = self.__awss3tool.get_object_byte(current_bucket_name,latest_package_key)
+            if result["success"] is False:
+                raise ValueError(f"download embedding package fail result {result}")
+            current_package_compress_byte = result["bytes"]
+            with open(current_feed_path,'wb') as f:
+                f.write(current_package_compress_byte)
+            self.__logger.debug(f'need_download {current_feed_path}')
+        else:
+            with open(current_feed_path,'rb') as f:
+                current_package_compress_byte = f.read()
+        
+        decompress_bytes = zlib.decompress(current_package_compress_byte)
+        current_latest_package = rec_proto_embebding.CategoryPackage()
+        current_latest_package.ParseFromString(decompress_bytes)
+        
+        all_category_dict = MessageToDict(current_latest_package,preserving_proto_field_name=True)
+        all_category_list = all_category_dict["category_list"]
+        category_name_to_category = dict()
+        for current_category in all_category_list:
+            current_category["id"] = int(current_category["id"])
+            category_name_to_category[current_category["category"]] = current_category
+        return category_name_to_category
\ No newline at end of file
